#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# @Author: José Sánchez-Gallego (gallegoj@uw.edu)
# @Date: 2023-10-24
# @Filename: build.py
# @License: BSD 3-clause (http://www.opensource.org/licenses/BSD-3-Clause)

from __future__ import annotations

import os
import shutil

from setuptools import Distribution, Extension
from setuptools.command.build_ext import build_ext


module = Extension(
    "src.tpm_multicast_client.tpmdgram",
    define_macros=[("MAJOR_VERSION", "1"), ("MINOR_VERSION", "0")],
    include_dirs=["cextern/include/"],
    sources=["cextern/src/tpmdgram.c", "cextern/src/tpmtable.c"],
    extra_link_args=["-fPIC"],
    language="c",
    optional=False,
)


# The *args is needed because the autogenerated setup.py sends arguments
# (this is a leftover from the old Poetry build system).
def build(*args, **setup_kwargs):
    distribution = Distribution({"name": "extended", "ext_modules": [module]})

    cmd = build_ext(distribution)
    cmd.ensure_finalized()
    cmd.run()

    # Copy built extensions back to the project
    for output in cmd.get_outputs():
        relative_extension = os.path.relpath(output, cmd.build_lib)
        shutil.copyfile(output, relative_extension)  # type: ignore
        mode = os.stat(relative_extension).st_mode
        mode |= (mode & 0o444) >> 2
        os.chmod(relative_extension, mode)


if __name__ == "__main__":
    build()
